;***************************************************************************
; 
; File Name		:'Main.asm"
; Title			:TriTone AFSK Modulator 
; Date			:2002.03.22.
; Version		:1.3.0
; Support telephone	:+36-70-333-4034
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AT90S2313
; 
;***************************************************************************
;	D E S C R I P T I O N
;
;  PD4: TTL Output for FSK
;  Rxd: FSK Data Input (1200 baud)
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date      who  	why
;	----	----------  ---		------------------------------------
;	0.01	2002.03.22  VFX		Creation
;	1.10	2002.04.17  VFX		TriTone Modulation 
;       1.30    2002.05.23  VFX		Negyszog jeles modulacio
;
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	Kvarc: f=4.000 MHz (T=125.00ns)
;*
;***************************************************************************
;
;
;
.include "2313def.inc"


;***************************************************************************
;* Const Def

;* fel periodusidok @4MHz:
;* 1200Hz  -> 1667 -> Low bit
;* 1500Hz  -> 1333 -> High bit
;*  960Hz  -> 2083 -> Guard (Tone)

.EQU	LowFrekv = 1667
.EQU	HiFrekv =  1333	
.EQU	GuardFrekv = 2083



;* UBR=INT(clk/(16*Baud)-1+0.5)

.EQU	BaudRate = 207		;1200 Baud @4.000MHz


;**************************************************************************
;* Hardware Def.


.EQU	FSKPort = PORTD
.EQU	FSKPortDIR = DDRD
.EQU	FSKBit = 4


;***************************************************************************
;**** VARIABLES
.DSEG

;***************************************************************************
.ESEG
		.DB 	"TriTone Modem [2002.03.22.] "
		.DB	"Latest Upd.: 2002.05.29."
		.DB	"Rev:1.3.0 Final"


;*****************************************************************************
;****		I N T E R R U P T S
;****
;*****************************************************************************
.CSEG
		rjmp RESET		;Reset Handle
		rjmp EXT_INTO		;IRQ0 Handle
		rjmp EXT_INT1		;IRQ1 Handle
		rjmp TIM_CAPT1		;Timer1 capture Handle
		rjmp TIM_COMP1		;Timer1 compare Handle
		rjmp TIM_OVF1		;Timer1 overflow Handle
		rjmp TIM_OVF0		;Timer0 overflow Handle
		rjmp UART_RXC		;UART RX Complete Handle
		rjmp UART_DRE		;UDR Empty Handle
		rjmp UART_TXC		;UART TX Complete Handle
		rjmp ANA_COMP		;Analog Comparator Handle

;********************************
;* RESET 
;********************************
RESET:		CLI			;GLOBAL INTERRUP DIS.	

		ldi R16, low(RAMEND)
		out SPL,R16

		LDI	R16,0b00101010	;Sleep enable, Idle mode,INT0, INT1 edge sens.
		OUT	MCUCR,R16	;7. = 1 SRE External SRAM Enabled
					;6. = 1 SRW Ext. SRAM Wait State Enabled
					;5. = 1 Sleep Enable
					;4. = 0 SLEEP => Idle Mode
					;     1 SLEEP => Power Down Mode
					;3:2 = Inerrupt Sense Control for INT1
					; 00 -> Low Level INT1
					; 01 -> Reserved
					; 10 -> Falling Edge INT1
					; 11 -> Rising Edge INT1
					;1:0 = Inerrupt Sense Control for INT0
					; 00 -> Low Level INT0
					; 01 -> Reserved
					; 10 -> Falling Edge INT0
					; 11 -> Rising Edge INT0



		LDI	R16,0b00000000	;INT0, INT1 Disabled
		OUT	GIMSK,R16	;7. = 1 EXT INT1 ACTIVE
					;6. = 1 EXT INT0 ACTIVE



		LDI	R16,0b01000000
		OUT	TIMSK,R16 	;7. = 1 TC1 OWRF EN.  *
					;6. = 1 TC1 COMP MATCH EN.
					;3. = 1 TC1 INP CAPTURE EN.
					;1. = 1 TC0 OWRF EN.   *

;************************************
;**  Timer0 Init		*****
;**
InitTimer0:	LDI	R20,GuardFrekv
		OUT	TCNT0,R16 
		LDI	R16,0b00000000
		OUT	TCCR0,R16	;TC0 Control Register
					;2:0 Clock Select
					; 000 -> Stop
					; 001 -> CK
					; 010 -> CK/8
					; 011 -> CK/64
					; 100 -> CK/256
					; 101 -> CK/1024
					; 110 -> External pin T0 falling edge
					; 111 -> External pin T0 risinrg edge


;*****************
; Timer1 Setup


		LDI	R17,0b00000000
		OUT	TCCR1A,R17	;Non-invert 8bit PWM, 
					;7:6 00 => OC1 not connected
					;    01 => OC1 not connected
					;    10 => OC1 non-inverted PWM
					;    11 => OC1 inverted PWM
					;1:0 00 => PWM Disabled
					;    01 =>  8 bit PWM
					;    10 =>  9 bit PWM
					;    11 => 10 bit PWM
       	
		LDI	R17,0b00001001
		OUT	TCCR1B,R17	;Input Capture Dis., f=CLK/8
					;7. = 0 Input Capture Noise Canceller Disabled
					;6. = 0 InputCapture ICP Falling edge
					;3. = 1 Reset TC1 when copare match
					;2:0 Clock select
					;   000 => TC1 Stop
					;   001 => Ck
					;   010 => Ck/8
					;   011 => Ck/64
					;   100 => Ck/256
					;   101 => Ck/1024
					;   110 => External T1 Falling Edge
					;   111 => External T1 rising Edge



;UART Init 
		LDI	R16,0b00011000		;RXCIE,TXCIE,UDRIE,RXEN,TXEN,CHR9,RXB8,TXB8
		OUT	UCR,R16
		LDI	R16,BaudRate
		OUT	UBRR,R16		;BaudRate Gen.
		ldi	R16,255
		out	USR,R16			;status torlese	

                                   
;FSK Port beallitas

		sbi	FSKPort,FSKBit		;kimeneti = 1
		sbi	FSKPortDIR,FSKBit	;minden bit output

		clr	R21			;bit counter lesz
		clr	R19			;nincs uj uart adat

		SEI				;SET GLOBAL INT EN.
		RJMP	Main


;*******************
EXT_INTO:	RETI
EXT_INT1:	RETI
TIM_CAPT1:	RETI
TIM_COMP1:	RETI
TIM_OVF1:	RETI

;********************
;* Timer0 Int
;* 
TIM_OVF0:	RETI

;*****************
;* UART RX COMPLET 
UART_RXC:	RETI

;*******************
UART_DRE:	RETI

;******************
;* UART TX COMPLET 
UART_TXC:	RETI

ANA_COMP:	RETI



;***********************************************************************
;***********************************************************************
;********     M     A      I      N				********
;********							********
;***********************************************************************
Main:

;elosszor 3 db guard jelet kell kikuldeni 

		ldi	R20,3

Guard3:		ldi	R16,High(GuardFrekv)
		out	OCR1AH,R16
		ldi	R16,Low(GuardFrekv)
		out	OCR1AL,R16
        	sleep
		nop

		cbi	FSKPort,FSKBit		;FSK = L
		sleep
		nop
		sbi	FSKPort,FSKBit		;FSK = H
		dec	R20
		brne	Guard3
						;Guard vege van uj adat ay UARTon?

		ldi	R20,1
		sbis	USR,RXC		;jott uj adat?
		  rjmp	Guard3
		
		in	R5,UDR		;R5=new data
		ldi	R19,255		;uj adat flag
		out	USR,R19		;status torlese	
		ldi	R21,8		;8 bitet kell majd kirakni

VanMegBit:
		ldi	ZL,Low(HiFrekv)
		ldi	ZH,High(HiFrekv)
                ror	R5		;0. bit lesz elkuldve
		brcs	MakeBit
		ldi	ZL,Low(LowFrekv)
		ldi	ZH,High(LowFrekv)

MakeBit:
		out	OCR1AH,ZH
		out	OCR1AL,ZL

		sleep
		nop
		cbi	FSKPort,FSKBit		;FSK = L
		sleep
		nop
		sbi	FSKPort,FSKBit		;FSK = H

		dec	R21
		brne	VanMegBit

                ldi	R20,2			;minden byte utan min. 2 db guard
		rjmp	Guard3
